﻿using GalaSoft.MvvmLight;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WpfApp2_pizza.VM
{
    /*
    * IEnumerable<T> => old-fashioned, not designed for UI
    *     IF the collection doesn't change after setting the DataContext
    * ObservableCollection<T> => for WPF, CollectionChanged event
    *      Notifies the UI when Add/Remove/Clear
    * BindingList<T> => for WPF, ListChanged event
    *      UI notification + event in code if a listitem changes
    */
    class PizzaVM : ObservableObject
    {
        public PizzaVM()
        {
            Toppings = new BindingList<PizzaExtraVM>();
            Size = new PizzaExtraVM();

            Toppings.ListChanged += (sender, evt) => RaisePropertyChanged(nameof(TotalPrice));
        }

        private PizzaExtraVM size;
        public PizzaExtraVM Size
        {
            get { return size; }
            set { Set(ref size, value); RaisePropertyChanged(nameof(TotalPrice)); }
        }
        public BindingList<PizzaExtraVM> Toppings { get; init; }

        public int TotalPrice
        {
            get
            {
                int price = Size.Price;
                foreach (PizzaExtraVM topping in Toppings)
                {
                    if (topping.IsWanted) price += topping.Price;
                }
                return price;
            }
        }

        public PizzaVM Clone() // DEEP COPY: Prototype design pattern!
        {
            return new PizzaVM()
            {
                Size = new PizzaExtraVM(Size),
                Toppings = new BindingList<PizzaExtraVM>(Toppings.
                    Where(x => x.IsWanted == true).
                    Select(x => new PizzaExtraVM(x)).
                    ToList())
            };
        }
    }
}
